/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
  * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.engvew.engstelle;

import static org.junit.Assert.assertTrue;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.vew.engvew.testtools.InputData;
import de.bsvrz.vew.engvew.testtools.JUnitHelper;

public class TestBaustelleFahrstreifen
{
  private static SystemObject _sysObjBaustelle = null;
  
  private static List<InputData> _baustellenData2 = new LinkedList<InputData>();
  
  private static List<InputData> _baustellenData4 = new LinkedList<InputData>();
      
  @BeforeClass
  public static void setUpBeforeClass() throws Exception 
  {
    _sysObjBaustelle = JUnitHelper.getInstance().erzeugeBaustelle();
          
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    Aspect asp = dm.getAspect("asp.parameterSoll");
    AttributeGroup atg = dm.getAttributeGroup("atg.situationsEigenschaften"); 
    DataDescription ddSituation =  new DataDescription(atg, asp);
    atg = dm.getAttributeGroup("atg.baustellenEigenschaften"); 
    DataDescription ddBaustelle =  new DataDescription(atg, asp);
    atg = dm.getAttributeGroup("atg.baustellenEigenschaftenErweitert"); 
    DataDescription ddErweitert =  new DataDescription(atg, asp);
    atg = dm.getAttributeGroup("atg.engstellenVerwaltungParameter");
    DataDescription ddParam =  new DataDescription(atg, asp);
  
    long aktuelleZeit = JUnitHelper.getInstance().getConnection().getTime();
  
    SystemObject mq2 = dm.getObject("mq.bw.mq_7_hfb1_sw");
    SystemObject ass2 = dm.getObject("ass.p.11908.p.11909.0");
    for (SystemObject fs: ((ConfigurationObject)mq2).getNonMutableSet("FahrStreifen").getElements())
    {
      // Fr die FS, die von _mq1 berwacht werden, herrscht VS 120 Fzg/h == 2 Fzg/min
      JUnitHelper.getInstance().setzeVerkehrstaerkeFs(fs, 120);
    }
            
    SystemObject mq4 = dm.getObject("mq.bw.mq_8_hfb1_sw");
    SystemObject ass4 = dm.getObject("ass.p.11909.p.11910.0");
    for (SystemObject fs: ((ConfigurationObject)mq4).getNonMutableSet("FahrStreifen").getElements())
    {
      // Fr die FS, die von _mq2 berwacht werden, herrscht VS 240 Fzg/h == 4 Fzg/min
      JUnitHelper.getInstance().setzeVerkehrstaerkeFs(fs, 240);
    }
    
    ResultData situation2, situation4, baustelle, erweitert, parameter;
    
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 300));
        
    /* Beginn Testflle VS auf Fahrstreifen ist 120 Fzg/h == 2 Fzg/min */
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxVsGesperrteFs(3)); // MaxVsGesperrteFs ist 3 Fzg/min
   
    situation2 = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass2 }, 100, 900));
    
    /* 1. Testfall:
     * Kein FS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(0, new String[] { }));
    _baustellenData2.add(new InputData(parameter, situation2, baustelle, erweitert, null));
    
    /* 2. Testfall:
     * HFS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(1, new String[] {"HFS"}));
    _baustellenData2.add(new InputData(parameter, situation2, baustelle, erweitert, null));
    
    /* 3. Testfall:
     * 1FS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(1, new String[] {"1FS"}));
    _baustellenData2.add(new InputData(parameter, situation2, baustelle, erweitert, null));
    
    /* 4. Testfall:
     * HFS und FS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(2, new String[] {"HFS", "1FS"}));
    _baustellenData2.add(new InputData(parameter, situation2, baustelle, erweitert, null));
    
    /* 5. Testfall:
     * HFS und FS bleiben gesperrt und Parameter MaxVsGesperrteFs wird auf 1 Fzg/min. gesetzt */
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxVsGesperrteFs(1)); // MaxVsGesperrteFs ist 1 Fzg/min
    _baustellenData2.add(new InputData(parameter, situation2, baustelle, erweitert, null));
    
    /* 6. Testfall:
     * 1FS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(1, new String[] {"1FS"}));
    _baustellenData2.add(new InputData(parameter, situation2, baustelle, erweitert, null));
    
    /* 7. Testfall:
     * HFS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(1, new String[] {"HFS"}));
    _baustellenData2.add(new InputData(parameter, situation2, baustelle, erweitert, null));
    
    /* 8. Testfall:
     * Kein FS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(0, new String[] { }));
    _baustellenData2.add(new InputData(parameter, situation2, baustelle, erweitert, null));    
    /* Ende Testflle VS auf Fahrstreifen ist 120 Fzg/h == 2 Fzg/min */

    /* Beginn Testflle VS auf Fahrstreifen ist 240 Fzg/h == 4 Fzg/min */
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxVsGesperrteFs(3)); // MaxVsGesperrteFs ist 3 Fzg/min
   
    situation4 = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass4 }, 100, 900)); 
    
    /* 1. Testfall:
     * Kein FS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(0, new String[] { }));
    _baustellenData4.add(new InputData(parameter, situation4, baustelle, erweitert, null));
    
    /* 2. Testfall:
     * HFS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(1, new String[] {"HFS"}));
    _baustellenData4.add(new InputData(parameter, situation4, baustelle, erweitert, null));
    
    /* 3. Testfall:
     * 1FS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(1, new String[] {"1FS"}));
    _baustellenData4.add(new InputData(parameter, situation4, baustelle, erweitert, null));
    
    /* 4. Testfall:
     * HFS und FS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(2, new String[] {"HFS", "1FS"}));
    _baustellenData4.add(new InputData(parameter, situation4, baustelle, erweitert, null));
    
    /* 5. Testfall:
     * HFS und FS bleiben gesperrt und Parameter MaxVsGesperrteFs wird auf 5 Fzg/min. gesetzt */
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxVsGesperrteFs(5)); // MaxVsGesperrteFs ist 5 Fzg/min
    _baustellenData4.add(new InputData(parameter, situation4, baustelle, erweitert, null));
    
    /* 6. Testfall:
     * 1FS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(1, new String[] {"1FS"}));
    _baustellenData4.add(new InputData(parameter, situation4, baustelle, erweitert, null));
    
    /* 7. Testfall:
     * HFS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(1, new String[] {"HFS"}));
    _baustellenData4.add(new InputData(parameter, situation4, baustelle, erweitert, null));
    
    /* 8. Testfall:
     * Kein FS gesperrt */
    erweitert = new ResultData(_sysObjBaustelle, ddErweitert, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaftenErweitert(0, new String[] { }));
    _baustellenData4.add(new InputData(parameter, situation4, baustelle, erweitert, null));    
    /* Ende Testflle VS auf Fahrstreifen ist 120 Fzg/h == 2 Fzg/min */
    
    JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_INIT);
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception 
  {
    JUnitHelper.getInstance().loescheBaustelle(_sysObjBaustelle);
    _sysObjBaustelle = null;
    _baustellenData2.clear();
    _baustellenData4.clear();
  }
  
  @Before
  public void setUp() throws Exception 
  {
   
  }

  @After
  public void tearDown() throws Exception 
  {
  
  }

  private String getFahrstreifenLage(SystemObject fs)
  {
    if (!fs.isOfType("typ.fahrStreifen"))
    {
      throw new IllegalArgumentException("Objekt vom Typ typ.fahrStreifen erwartet");
    }
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    AttributeGroup atg = dm.getAttributeGroup("atg.fahrStreifen");
    String lage = fs.getConfigurationData(atg).getItem("Lage").asTextValue().getText();
    return lage == null ? "" : lage;
  }
  
  @Test
  public final void testpruefeGesperrteFahrstreifenVS2() 
  {
    Baustelle baustelle = null;
    for (Situation lauf: JUnitHelper.getInstance().getEngVew().getVerwalteteObjekte())
    {
      if (lauf.getObjekt().equals(_sysObjBaustelle))
      {
        baustelle = (Baustelle)lauf;
        break;
      }
    }
    assertTrue("Baustelle wird verwaltet", baustelle != null &&
        baustelle.getObjekt() == _sysObjBaustelle);
       
    int laufendeTestNr = 0;
    
    for (InputData inputData: _baustellenData2)
    {
      baustelle.update(new ResultData[] { inputData.situationsEigenschaften });
      baustelle.update(new ResultData[] { inputData.baustellenEigenschaften });
      baustelle.update(new ResultData[] { inputData.baustellenEigenschaftenErweitert });
      JUnitHelper.getInstance().getEngVew().update(new ResultData[] { inputData.engstellenVerwaltung });
      laufendeTestNr += 1;
      
      JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_UPDATE);
      
      Collection<ErgebnisFsPruefung> result = baustelle.pruefeGesperrteFahrstreifen();
      
      if (laufendeTestNr == 1)
      {
        assertTrue("Prfung gesperrte Fahrstreifen nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 2)
      {
        assertTrue("Prfung gesperrte Fahrstreifen nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 3)
      {
        assertTrue("Prfung gesperrte Fahrstreifen nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 4)
      {
        assertTrue("Prfung gesperrte Fahrstreifen nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 5)
      {
        assertTrue("Prfung gesperrte Fahrstreifen fr 2 Fahrstreifen verletzt", result.size() == 2);
        Set<String> betroffeneFahrstreifen = new HashSet<String>();
        for (ErgebnisFsPruefung ergebnis: result)
        {
          betroffeneFahrstreifen.add(getFahrstreifenLage(ergebnis.fs));
        }
        assertTrue("Betroffener Fahrstreifen ist HFS", betroffeneFahrstreifen.contains("HFS")); 
        assertTrue("Betroffener Fahrstreifen ist 1FS", betroffeneFahrstreifen.contains("1FS"));
      }
      if (laufendeTestNr == 6)
      {
        assertTrue("Prfung gesperrte Fahrstreifen fr 1 Fahrstreifen verletzt", result.size() == 1);
        assertTrue("Betroffener Fahrstreifen ist 1FS", 
            getFahrstreifenLage(result.iterator().next().fs).equals("1FS"));
      }      
      if (laufendeTestNr == 7)
      {
        assertTrue("Prfung gesperrte Fahrstreifen fr 1 Fahrstreifen verletzt", result.size() == 1);
        assertTrue("Betroffener Fahrstreifen ist HFS", 
            getFahrstreifenLage(result.iterator().next().fs).equals("HFS"));
      }      
      if (laufendeTestNr == 8)
      {
        assertTrue("Prfung gesperrte Fahrstreifen nicht verletzt", result.isEmpty());
      }
      
      System.out.println("Prfung gesperrte FS (VS 2 Fzg/min) Nr. " + laufendeTestNr + ": OK");
    }
  }
  
  @Test
  public final void testpruefeGesperrteFahrstreifenVS4() 
  {
    Baustelle baustelle = null;
    for (Situation lauf: JUnitHelper.getInstance().getEngVew().getVerwalteteObjekte())
    {
      if (lauf.getObjekt().equals(_sysObjBaustelle))
      {
        baustelle = (Baustelle)lauf;
        break;
      }
    }
    assertTrue("Baustelle wird verwaltet", baustelle != null &&
        baustelle.getObjekt() == _sysObjBaustelle);
       
    int laufendeTestNr = 0;
    
    for (InputData inputData: _baustellenData4)
    {
      baustelle.update(new ResultData[] { inputData.situationsEigenschaften });
      baustelle.update(new ResultData[] { inputData.baustellenEigenschaften });
      baustelle.update(new ResultData[] { inputData.baustellenEigenschaftenErweitert });
      JUnitHelper.getInstance().getEngVew().update(new ResultData[] { inputData.engstellenVerwaltung });
      laufendeTestNr += 1;
      
      JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_UPDATE);
      
      Collection<ErgebnisFsPruefung> result = baustelle.pruefeGesperrteFahrstreifen();
            
      if (laufendeTestNr == 1)
      {
        assertTrue("Prfung gesperrte Fahrstreifen nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 2)
      {
        assertTrue("Prfung gesperrte Fahrstreifen fr 1 Fahrstreifen verletzt", result.size() == 1);
        assertTrue("Betroffener Fahrstreifen ist HFS", 
            getFahrstreifenLage(result.iterator().next().fs).equals("HFS"));
      }
      if (laufendeTestNr == 3)
      {
        assertTrue("Prfung gesperrte Fahrstreifen fr 1 Fahrstreifen verletzt", result.size() == 1);
        assertTrue("Betroffener Fahrstreifen ist 1FS", 
            getFahrstreifenLage(result.iterator().next().fs).equals("1FS"));
      }
      if (laufendeTestNr == 4)
      {
        assertTrue("Prfung gesperrte Fahrstreifen fr 2 Fahrstreifen verletzt", result.size() == 2);
        Set<String> betroffeneFahrstreifen = new HashSet<String>();
        for (ErgebnisFsPruefung ergebnis: result)
        {
          betroffeneFahrstreifen.add(getFahrstreifenLage(ergebnis.fs));
        }
        assertTrue("Betroffener Fahrstreifen ist HFS", betroffeneFahrstreifen.contains("HFS")); 
        assertTrue("Betroffener Fahrstreifen ist 1FS", betroffeneFahrstreifen.contains("1FS"));
      }
      if (laufendeTestNr == 5)
      {
        assertTrue("Prfung gesperrte Fahrstreifen nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 6)
      {
        assertTrue("Prfung gesperrte Fahrstreifen nicht verletzt", result.isEmpty());
      }      
      if (laufendeTestNr == 7)
      {
        assertTrue("Prfung gesperrte Fahrstreifen nicht verletzt", result.isEmpty());
      }      
      if (laufendeTestNr == 8)
      {
        assertTrue("Prfung gesperrte Fahrstreifen nicht verletzt", result.isEmpty());
      }
      
      System.out.println("Prfung gesperrte FS (VS 4 Fzg/min) Nr. " + laufendeTestNr + ": OK");
    }
  }
  
}
