/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.engvew.engstelle;

import static org.junit.Assert.assertTrue;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.vew.engvew.testtools.InputData;
import de.bsvrz.vew.engvew.testtools.JUnitHelper;

public class TestBaustelleStau
{

  private static SystemObject _sysObjBaustelle = null;
  private static SystemObject _sysObjStau = null;
  
  private static List<InputData> _baustellenData = new LinkedList<InputData>();
  
  private static   List<InputData> _stauData = new LinkedList<InputData>();
  
  private static   InputData _baustelleFix = null;
  
  private static   InputData _stauFix = null;
    
  @BeforeClass
  public static void setUpBeforeClass() throws Exception 
  {
    _sysObjBaustelle = JUnitHelper.getInstance().erzeugeBaustelle();
    _sysObjStau = JUnitHelper.getInstance().erzeugeStau();
    
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    Aspect asp = dm.getAspect("asp.parameterSoll");
    AttributeGroup atg = dm.getAttributeGroup("atg.situationsEigenschaften"); 
    DataDescription ddSituation =  new DataDescription(atg, asp);
    atg = dm.getAttributeGroup("atg.baustellenEigenschaften"); 
    DataDescription ddBaustelle =  new DataDescription(atg, asp);
    atg = dm.getAttributeGroup("atg.engstellenVerwaltungParameter");
    DataDescription ddParam =  new DataDescription(atg, asp);
  
    long aktuelleZeit = JUnitHelper.getInstance().getConnection().getTime();
    SystemObject ass = dm.getObject("ass.p.11909.p.11910.0");
    
    ResultData baustelleFixSituation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0, 
        new SystemObject[] { ass }, 500, 1500));
    ResultData baustelleFixBaustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1000));
    _baustelleFix = new InputData(null, baustelleFixSituation, baustelleFixBaustelle, null, null);
    
    ResultData stauFix = new ResultData(_sysObjStau, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0, 
            new SystemObject[] { ass }, 500, 1500));
    _stauFix = new InputData(null, stauFix, null, null, null);
    
    ResultData situation, baustelle, parameter; 
    /* Beginn Testflle mit Stau fix, Baustelle variabel */
    /* 1. Testfall:
     * Baustelle endet 10m vor Stauende */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 200, 1490));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1000));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxLaengeBaustellenEnde(20)); // Max. Lnge ber Baustellenende ist 20m
    _baustellenData.add(new InputData(parameter, situation, baustelle, null, null));
    
    /* 2. Testfall:
     * Baustelle endet 30m vor Stauende */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 200, 1470));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1000));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxLaengeBaustellenEnde(20)); // Max. Lnge ber Baustellenende ist 20m
    _baustellenData.add(new InputData(parameter, situation, baustelle, null, null));
    
    /* 3. Testfall:
     * Baustelle endet 10m vor Stauende, aber Parameter max. Lnge ber Baustellenende 
     * ist 5m */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 200, 1490));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1000));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxLaengeBaustellenEnde(5)); // Max. Lnge ber Baustellenende ist 5m
    _baustellenData.add(new InputData(parameter, situation, baustelle, null, null));

    /* 4. Testfall:
     * Baustelle endet 30m vor Stauende, aber Parameter max. Lnge ber Baustellenende 
     * ist 35m */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 200, 1470));
    baustelle = new ResultData(_sysObjBaustelle, ddBaustelle, aktuelleZeit, 
        JUnitHelper.getInstance().getDataBaustellenEigenschaften("gltig", 1000));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxLaengeBaustellenEnde(35)); // Max. Lnge ber Baustellenende ist 35m
    _baustellenData.add(new InputData(parameter, situation, baustelle, null, null));
    /* Ende Testflle mit Stau fix, Baustelle variabel */
    
    /* Beginn Testflle mit Baustelle fix, Stau variabel */
    /* 1. Testfall:
     * Stau geht 10m ber Baustellenende */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 500, 1510));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxLaengeBaustellenEnde(20)); // Max. Lnge ber Baustellenende ist 20m
    _stauData.add(new InputData(parameter, situation, baustelle, null, null));
    
    /* 2. Testfall:
     * Stau geht 30m ber Baustellenende */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 200, 1530));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxLaengeBaustellenEnde(20)); // Max. Lnge ber Baustellenende ist 20m
    _stauData.add(new InputData(parameter, situation, baustelle, null, null));
    
    /* 3. Testfall:
     * Stau geht 10m ber Baustellenende, aber Parameter max. Lnge ber Baustellenende 
     * ist 5m */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 200, 1510));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxLaengeBaustellenEnde(5)); // Max. Lnge ber Baustellenende ist 5m
    _stauData.add(new InputData(parameter, situation, baustelle, null, null));

    /* 4. Testfall:
     * Stau geht 30m ber Baustellenende, aber Parameter max. Lnge ber Baustellenende 
     * ist 35m */
    situation = new ResultData(_sysObjBaustelle, ddSituation, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit, 0,
        new SystemObject[] { ass }, 200, 1530));
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataMaxLaengeBaustellenEnde(35)); // Max. Lnge ber Baustellenende ist 35m
    _stauData.add(new InputData(parameter, situation, baustelle, null, null));
    /* Ende Testflle mit Stau fix, Baustelle variabel */
    
    JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_INIT);
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception 
  {
    JUnitHelper.getInstance().loescheBaustelle(_sysObjBaustelle);
    JUnitHelper.getInstance().loescheStau(_sysObjStau);
    _sysObjBaustelle = null;
    _sysObjStau = null;
    _baustellenData.clear();
    _stauData.clear();
    _baustelleFix = null;
    _stauFix = null;
  }
  
  @Before
  public void setUp() throws Exception 
  {
   
  }

  @After
  public void tearDown() throws Exception 
  {
  
  }

  @Test
  public final void testPruefeStaus_StauFix() 
  {
    Baustelle baustelle = null;
    for (Situation lauf: JUnitHelper.getInstance().getEngVew().getVerwalteteObjekte())
    {
      if (lauf.getObjekt().equals(_sysObjBaustelle))
      {
        baustelle = (Baustelle)lauf;
        break;
      }
    }
    assertTrue("Baustelle wird verwaltet", baustelle != null &&
        baustelle.getObjekt() == _sysObjBaustelle);
    
    Stau stau = null;
    for (Situation lauf: JUnitHelper.getInstance().getEngVew().getVerwaltungsStaus().getVerwalteteObjekte())
    {
      if (lauf.getObjekt().equals(_sysObjStau))
      {
        stau = (Stau)lauf;
        break;
      }
    }
    assertTrue("Stau wird verwaltet", stau != null &&
        stau.getObjekt() == _sysObjStau);

    stau.update(new ResultData[] { _stauFix.situationsEigenschaften });
    int laufendeTestNr = 0;
    
    for (InputData inputData: _baustellenData)
    {
      baustelle.update(new ResultData[] { inputData.situationsEigenschaften });
      baustelle.update(new ResultData[] { inputData.baustellenEigenschaften });
      JUnitHelper.getInstance().getEngVew().update(new ResultData[] { inputData.engstellenVerwaltung });
      
      JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_UPDATE);
      
      laufendeTestNr += 1;
      
      Collection<ErgebnisStauPruefung> result = baustelle.pruefeStaus();
            
      if (laufendeTestNr == 1)
      {
        assertTrue("Prfung Staulnge ber Baustellenende nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 2)
      {
        assertTrue("Stau geht um 30m ber Baustellenende", result.size() == 1 &&
            result.iterator().next().stau == _sysObjStau &&
            result.iterator().next().ueberLaenge == 30.0);
      }
      if (laufendeTestNr == 3)
      {
        assertTrue("Stau geht um 10m ber Baustellenende", result.size() == 1 &&
            result.iterator().next().stau == _sysObjStau &&
            result.iterator().next().ueberLaenge == 10.0);
      }
      if (laufendeTestNr == 4)
      {
        assertTrue("Prfung Staulnge ber Baustellenende nicht verletzt", result.isEmpty());
      }
      
      System.out.println("Prfung Stauende ber Baustelle (Baustelle variabel) Nr. " + laufendeTestNr + ": OK");
    }
  }
  
  @Test
  public final void testPruefeStaus_BaustelleFix() 
  {
    Baustelle baustelle = null;
    for (Situation lauf: JUnitHelper.getInstance().getEngVew().getVerwalteteObjekte())
    {
      if (lauf.getObjekt().equals(_sysObjBaustelle))
      {
        baustelle = (Baustelle)lauf;
        break;
      }
    }
    assertTrue("Baustelle wird verwaltet", baustelle != null &&
        baustelle.getObjekt() == _sysObjBaustelle);
    
    Stau stau = null;
    for (Situation lauf: JUnitHelper.getInstance().getEngVew().getVerwaltungsStaus().getVerwalteteObjekte())
    {
      if (lauf.getObjekt().equals(_sysObjStau))
      {
        stau = (Stau)lauf;
        break;
      }
    }
    assertTrue("Stau wird verwaltet", stau != null &&
        stau.getObjekt() == _sysObjStau);

    baustelle.abmelden();
    stau.abmelden();
    baustelle.update(new ResultData[] { _baustelleFix.situationsEigenschaften });
    baustelle.update(new ResultData[] { _baustelleFix.baustellenEigenschaften });
    
    JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_UPDATE);
    
    int laufendeTestNr = 0;
    
    for (InputData inputData: _stauData)
    {
      stau.update(new ResultData[] { inputData.situationsEigenschaften });
      JUnitHelper.getInstance().getEngVew().update(new ResultData[] { inputData.engstellenVerwaltung });
      laufendeTestNr += 1;
      
      Collection<ErgebnisStauPruefung> result = baustelle.pruefeStaus();
            
      if (laufendeTestNr == 1)
      {
        assertTrue("Prfung Staulnge ber Baustellenende nicht verletzt", result.isEmpty());
      }
      if (laufendeTestNr == 2)
      {
        assertTrue("Stau geht um 30m ber Baustellenende", result.size() == 1 &&
            result.iterator().next().stau == _sysObjStau &&
            result.iterator().next().ueberLaenge == 30.0);
      }
      if (laufendeTestNr == 3)
      {
        assertTrue("Stau geht um 10m ber Baustellenende", result.size() == 1 &&
            result.iterator().next().stau == _sysObjStau &&
            result.iterator().next().ueberLaenge == 10.0);
      }
      if (laufendeTestNr == 4)
      {
        assertTrue("Prfung Staulnge ber Baustellenende nicht verletzt", result.isEmpty());
      }
      
      System.out.println("Prfung Stauende ber Baustelle (Stau variabel) Nr. " + laufendeTestNr + ": OK");
    }
  }

}
